namespace FrameworkQ.ObjectSpace.Data.SqlObjects
{
	/// <summary>
	/// A collection of elements of type OrderByTerm
	/// </summary>
	public class OrderByItemCollection : System.Collections.CollectionBase
	{
		/// <summary>
		/// Initializes a new empty instance of the OrderByTermCollection class.
		/// </summary>
		public OrderByItemCollection()
		{
			// empty
		}

		/// <summary>
		/// Initializes a new instance of the OrderByTermCollection class, containing elements
		/// copied from an array.
		/// </summary>
		/// <param name="items">
		/// The array whose elements are to be added to the new OrderByTermCollection.
		/// </param>
		public OrderByItemCollection(OrderByItem[] items)
		{
			this.AddRange(items);
		}

		/// <summary>
		/// Initializes a new instance of the OrderByTermCollection class, containing elements
		/// copied from another instance of OrderByTermCollection
		/// </summary>
		/// <param name="items">
		/// The OrderByTermCollection whose elements are to be added to the new OrderByTermCollection.
		/// </param>
		public OrderByItemCollection(OrderByItemCollection items)
		{
			this.AddRange(items);
		}

		/// <summary>
		/// Adds the elements of an array to the end of this OrderByTermCollection.
		/// </summary>
		/// <param name="items">
		/// The array whose elements are to be added to the end of this OrderByTermCollection.
		/// </param>
		public virtual void AddRange(OrderByItem[] items)
		{
			foreach (OrderByItem item in items)
			{
				this.List.Add(item);
			}
		}

		/// <summary>
		/// Adds the elements of another OrderByTermCollection to the end of this OrderByTermCollection.
		/// </summary>
		/// <param name="items">
		/// The OrderByTermCollection whose elements are to be added to the end of this OrderByTermCollection.
		/// </param>
		public virtual void AddRange(OrderByItemCollection items)
		{
			foreach (OrderByItem item in items)
			{
				this.List.Add(item);
			}
		}

		/// <summary>
		/// Adds an instance of type OrderByTerm to the end of this OrderByTermCollection.
		/// </summary>
		/// <param name="value">
		/// The OrderByTerm to be added to the end of this OrderByTermCollection.
		/// </param>
		public virtual void Add(OrderByItem value)
		{
			this.List.Add(value);
		}

		/// <summary>
		/// Determines whether a specfic OrderByTerm value is in this OrderByTermCollection.
		/// </summary>
		/// <param name="value">
		/// The OrderByTerm value to locate in this OrderByTermCollection.
		/// </param>
		/// <returns>
		/// true if value is found in this OrderByTermCollection;
		/// false otherwise.
		/// </returns>
		public virtual bool Contains(OrderByItem value)
		{
			return this.List.Contains(value);
		}

		/// <summary>
		/// Return the zero-based index of the first occurrence of a specific value
		/// in this OrderByTermCollection
		/// </summary>
		/// <param name="value">
		/// The OrderByTerm value to locate in the OrderByTermCollection.
		/// </param>
		/// <returns>
		/// The zero-based index of the first occurrence of the _ELEMENT value if found;
		/// -1 otherwise.
		/// </returns>
		public virtual int IndexOf(OrderByItem value)
		{
			return this.List.IndexOf(value);
		}

		/// <summary>
		/// Inserts an element into the OrderByTermCollection at the specified index
		/// </summary>
		/// <param name="index">
		/// The index at which the OrderByTerm is to be inserted.
		/// </param>
		/// <param name="value">
		/// The OrderByTerm to insert.
		/// </param>
		public virtual void Insert(int index, OrderByItem value)
		{
			this.List.Insert(index, value);
		}

		/// <summary>
		/// Gets or sets the OrderByTerm at the given index in this OrderByTermCollection.
		/// </summary>
		public virtual OrderByItem this[int index]
		{
			get { return (OrderByItem) this.List[index]; }
			set { this.List[index] = value; }
		}

		/// <summary>
		/// Removes the first occurrence of a specific OrderByTerm from this OrderByTermCollection.
		/// </summary>
		/// <param name="value">
		/// The OrderByTerm value to remove from this OrderByTermCollection.
		/// </param>
		public virtual void Remove(OrderByItem value)
		{
			this.List.Remove(value);
		}

		/// <summary>
		/// Type-specific enumeration class, used by OrderByTermCollection.GetEnumerator.
		/// </summary>
		public class Enumerator : System.Collections.IEnumerator
		{
			private System.Collections.IEnumerator wrapped;

			/// <summary>
			/// 
			/// </summary>
			/// <param name="collection"></param>
			public Enumerator(OrderByItemCollection collection)
			{
				this.wrapped = ((System.Collections.CollectionBase) collection).GetEnumerator();
			}

			/// <summary>
			/// 
			/// </summary>
			public OrderByItem Current
			{
				get { return (OrderByItem) (this.wrapped.Current); }
			}

			object System.Collections.IEnumerator.Current
			{
				get { return (OrderByItem) (this.wrapped.Current); }
			}

			/// <summary>
			/// 
			/// </summary>
			/// <returns></returns>
			public bool MoveNext()
			{
				return this.wrapped.MoveNext();
			}

			/// <summary>
			/// 
			/// </summary>
			public void Reset()
			{
				this.wrapped.Reset();
			}
		}

		/// <summary>
		/// Returns an enumerator that can iterate through the elements of this OrderByTermCollection.
		/// </summary>
		/// <returns>
		/// An object that implements System.Collections.IEnumerator.
		/// </returns>        
		public new virtual OrderByItemCollection.Enumerator GetEnumerator()
		{
			return new OrderByItemCollection.Enumerator(this);
		}
	}
}